import {
  select,
  csv,
  scaleLinear,
  max,
  scaleBand,
  axisLeft,
  axisBottom,
  format,
} from 'd3';

const svg = select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');

const render = (data) => {
  // value accessors:
  const xValue = (d) => d.population;
  const yValue = (d) => d.country;
  // adding margins:
  const margin = {
    top: 55,
    right: 40,
    bottom: 70,
    left: 170,
  };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;

  // setting up linear and band scales:
  const xScale = scaleLinear() // we can envoke scale linear as a function to create an instance of a d3 scale
    .domain([0, max(data, xValue)])
    .range([0, innerWidth]); // this means the maximum population should map to the maximum bar width
  // set a y scale too
  const yScale = scaleBand()
    .domain(data.map(yValue))
    .range([0, innerHeight]) // allows the data elements to be arranged from top to bottom
    .padding(0.1);

  // lets append our rectangles to g instead of svg so that we have a margin...
  const g = svg
    .append('g')
    .attr(
      'transform',
      `translate(${margin.left},${margin.top})`,
    );

  // since billions for the SI standard is G..
  //but we want it to say B so lets create a format function...
  const xAxisTickFormat = (number) =>
    format('.3s')(number).replace('G', 'B');

  // setting up axes:
  const xAxis = axisBottom(xScale)
    .tickFormat(xAxisTickFormat) // formats the x tic labels
    .tickSize(-innerHeight); // create vertical lines...

  g.append('g')
    .call(axisLeft(yScale))
    .selectAll('.domain,.tick line') // get the domain class from this group tag, get the tick class, and then get its  nested line element
    .remove(); // remove it.. ( to remove the y axis line)
  // select all starts at the class level... <g class ="domain"...>  or <g class = tick...>

  const xAxisG = g // made xAxisG so that we can access this group
    .append('g')
    .call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);

  xAxisG
    .select('.domain') // get the domain class from this group tag
    .remove();

  // making x axis labels
  xAxisG
    .append('text')
    .attr('class', 'x-axis-label')
    .attr('fill', 'black') // make the text appear
    .attr('y', 60)
    .attr('x', innerWidth / 2)
    .text('Population');

  // drawing the rectangles:
  g.selectAll('rect')
    .data(data) // d3 selects all existing rectangles ( this is the d3 data join)
    .enter()
    .append('rect')
    .attr('y', (d) => yScale(yValue(d)))
    .attr('width', (d) => xScale(xValue(d)))
    .attr('height', yScale.bandwidth());

  // add title
  g.append('text')
    .attr('class', 'title')
    .attr('y', -10)
    .text('Top Ten Most popular Countries');
};

// loading and parsing data:
csv('data.csv').then((data) => {
  data.forEach((d) => {
    d.population = +d.population * 1000;
  });
  render(data);
});

//  csv() is a promis that resolves when this data is loaded
//  this csv() function of D3 makes an http request ,
//  loads the csv string from data.csv
//  then parses the csv into an array of objects

//keys of each object represent the columns
// values represent the values in the table
